import { createSignal } from "solid-js";
import { localState } from "./state";

export interface User {
    username: string,
    displayName: string,
    googleId: string,
}

export interface AuthenticationResponse {
    user: User,
    token: string,
}

export interface AuthConfig {
    clientId: string,
    redirect_url: string,
}

export class API {
    public apiURL: string;

    constructor(apiURL: string) {
        this.apiURL = apiURL;
    }

    private async checkResponse(response: Response) {
        if (!response.ok) {
            let json = null;
            try {
                json = await response.json();
            } catch (e) { /* ignored */ }

            throw json?.error || 'Request failed: ' + response.status + ' ' + response.statusText;
        }
    }

    public ensureLoggedIn(): AuthenticationResponse {
        const info = localState.accountInfo;
        if (info == null) throw 'Not logged in';
        return info;
    }

    public async authConfig(): Promise<AuthConfig> {
        const response = await fetch(this.apiURL + '/config');

        await this.checkResponse(response);

        return await response.json();
    }

    public async login(code: string): Promise<AuthenticationResponse> {
        const response = await fetch(this.apiURL + '/auth/login', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({ code })
        });

        await this.checkResponse(response);

        return await response.json();
    }
}